@@ -1,10 +1,10 @@
 #!/usr/bin/env python
 from . import process_latex
 from . import process_text
-from .config import math_code
 from .process_text import char_limit
 import time
 import re
+import tqdm
 
 default_begin = r'''
 \documentclass[UTF8]{article}
@@ -16,6 +16,10 @@ default_end = r'''
 \end{document}
 '''
 
+# TODO: add more here
+environment_list = ['abstract', 'acknowledgments', 'itemize', 'enumerate', 'description', 'list', 'proof']
+command_list = ['section', 'subsection', 'subsubsection', 'caption', 'subcaption', 'footnote', 'paragraph']
+
 
 class TextTranslator:
     def __init__(self, engine, language_to, language_from):
@@ -43,8 +47,7 @@ class TextTranslator:
                 break
             except BaseException as e:
                 if hasattr(self.translator, "is_error_request_frequency") and self.translator.is_error_request_frequency(e):
-                    print("sleep 1 second to wait")
-                    time.sleep(1)
+                    time.sleep(0.5)
                 else:
                     raise e
         return result
@@ -88,29 +91,33 @@ class LatexTranslator:
         text_translated = '\n'.join(parts_translated)
         return text_translated.replace("\u200b", "")
 
-    def translate_paragraph_latex(self, latex_original_paragraph, num, complete):
+    def translate_paragraph_latex(self, latex_original_paragraph):
         '''
         Translate a latex paragraph, which means that it could contain latex objects
         '''
         text_original_paragraph, objs = process_latex.replace_latex_objects(latex_original_paragraph)
         # Since \n is equivalent to space in latex, we change \n back to space
         # otherwise the translators view them as separate sentences
-        text_original_paragraph = text_original_paragraph.replace('\n', '')
+        text_original_paragraph = process_latex.combine_split_to_sentences(text_original_paragraph)
         text_original_paragraph = process_text.split_too_long_paragraphs(text_original_paragraph)
-        if not complete:
+        if not self.complete:
             text_original_paragraph = process_text.split_titles(text_original_paragraph)
+        # Remove additional space
+        text_original_paragraph = re.sub(r'  +', ' ', text_original_paragraph)
         if self.debug:
-            print(f'\n\nParagraph {num}\n\n', file=self.f_old)
+            print(f'\n\nParagraph {self.num}\n\n', file=self.f_old)
             print(text_original_paragraph, file=self.f_old)
         text_translated_paragraph = self.translate_paragraph_text(text_original_paragraph)
         if self.debug:
-            print(f'\n\nParagraph {num}\n\n', file=self.f_new)
+            print(f'\n\nParagraph {self.num}\n\n', file=self.f_new)
             print(text_translated_paragraph, file=self.f_new)
-            print(f'\n\nParagraph {num}\n\n', file=self.f_obj)
+            print(f'\n\nParagraph {self.num}\n\n', file=self.f_obj)
             for i, obj in enumerate(objs):
                 print(f'obj {i}', file=self.f_obj)
                 print(obj, file=self.f_obj)
-        latex_translated_paragraph = process_latex.recover_latex_objects(text_translated_paragraph, objs, final=True)
+        latex_translated_paragraph, nbad, ntotal = process_latex.recover_latex_objects(text_translated_paragraph, objs, final=True)
+        self.nbad += nbad
+        self.ntotal += ntotal
         return latex_translated_paragraph
 
     def split_latex_to_paragraphs(self, latex):
@@ -121,47 +128,36 @@ class LatexTranslator:
         '''
         text, objs = process_latex.replace_latex_objects(latex)
         paragraphs_text = re.split(r'\n\n+', text)
-        paragraphs_latex = [process_latex.recover_latex_objects(paragraph_text, objs) for paragraph_text in paragraphs_text]
+        paragraphs_latex = [process_latex.recover_latex_objects(paragraph_text, objs)[0] for paragraph_text in paragraphs_text]
         return paragraphs_latex
 
-    def _translate_latex_objects(self, match_function, latex_original, names, complete):
+    def translate_latex_all_objects(self, latex, envs, commands):
         '''
         Terminology:
         env: '\\begin{xxx} \\end{xxx}'
         command: '\\command[options]{text}
         object: env or command
         '''
-        latex_translated = latex_original
+        for env_name in envs:
+            latex = process_latex.process_specific_env(latex, self.translate_paragraph_latex, env_name)
+            latex = process_latex.process_specific_env(latex, self.translate_paragraph_latex, env_name + r'\*')
+        for command_name in commands:
+            latex = process_latex.process_specific_command(latex, self.translate_paragraph_latex, command_name)
+            latex = process_latex.process_specific_command(latex, self.translate_paragraph_latex, command_name + r'\*')
+        return latex
 
-        num = 0
+    def translate_full_latex(self, latex_original):
+        self.nbad = 0
+        self.ntotal = 0
 
-        def translate_function(latex):
-            # Translate anything inside an environment or command
-            nonlocal num
-            result = self.translate_paragraph_latex(latex, num, complete)
-            num += 1
-            print(num)
-            return result
-
-        names = names + [item + '*' for item in names]
-        latex_translated = match_function(latex_translated, translate_function, names)
-        return latex_translated
-
-    def translate_latex_env(self, latex_original, names, complete):
-        return self._translate_latex_objects(process_latex.process_specific_env, latex_original, names, complete)
-
-    def translate_latex_commands(self, latex_original, names, complete):
-        return self._translate_latex_objects(process_latex.process_specific_commands, latex_original, names, complete)
-
-    def translate_full_latex(self, latex_original, loadmain=False):
         latex_original = process_latex.remove_tex_comments(latex_original)
 
         latex_original = process_latex.replace_accent(latex_original)
         latex_original = process_latex.replace_special(latex_original)
 
-        complete = process_latex.is_complete(latex_original)
+        self.complete = process_latex.is_complete(latex_original)
         theorems = process_latex.get_theorems(latex_original)
-        if complete:
+        if self.complete:
             print('It is a full latex document')
             latex_original, tex_begin, tex_end = process_latex.split_latex_document(latex_original, r'\begin{document}', r'\end{document}')
             tex_begin = process_latex.remove_blank_lines(tex_begin)
@@ -173,42 +169,35 @@ class LatexTranslator:
             tex_begin = default_begin
             tex_end = default_end
 
-        if loadmain:
-            latex_translated = open('text_after_main.txt').read()
-        else:
-            latex_original_paragraphs = self.split_latex_to_paragraphs(latex_original)
-            latex_translated_paragraphs = []
+        latex_original_paragraphs = self.split_latex_to_paragraphs(latex_original)
+        latex_translated_paragraphs = []
 
-            num = 0
-            print('processing main text')
-            for latex_original_paragraph in latex_original_paragraphs:
-                latex_translated_paragraph = self.translate_paragraph_latex(latex_original_paragraph, num, complete)
+        self.num = 0
+        for latex_original_paragraph in tqdm.tqdm(latex_original_paragraphs):
+            try:
+                latex_translated_paragraph = self.translate_paragraph_latex(latex_original_paragraph)
+                latex_translated_paragraph = self.translate_latex_all_objects(latex_translated_paragraph, environment_list + theorems, command_list)
                 latex_translated_paragraphs.append(latex_translated_paragraph)
-                print(num, '/', len(latex_original_paragraphs))
-                num += 1
+            except BaseException as e:
+                print('Error found in Parapragh', self.num)
+                print('Content')
+                print(latex_original_paragraph)
+                raise e
+            self.num += 1
 
-            latex_translated = '\n\n'.join(latex_translated_paragraphs)
-
-            if self.debug:
-                print(latex_translated, file=open('text_after_main.txt', 'w'))
-
-        # TODO: add more here
-        environment_list = ['abstract', 'acknowledgments', 'itemize', 'enumerate', 'description', 'list', 'proof']
-        print('processing latex environments')
-        latex_translated = self.translate_latex_env(latex_translated, environment_list + theorems, complete)
-
-        command_list = ['section', 'subsection', 'subsubsection', 'caption', 'subcaption', 'footnote', 'paragraph']
-        print('processing latex commands')
-        latex_translated = self.translate_latex_commands(latex_translated, command_list, complete)
+        latex_translated = '\n\n'.join(latex_translated_paragraphs)
 
         latex_translated = tex_begin + '\n' + latex_translated + '\n' + tex_end
 
         # Title is probably outside the body part
-        print('processing title')
-        latex_translated = self.translate_latex_commands(latex_translated, ['title'], complete)
+        self.num = 'title'
+        latex_translated = process_latex.process_specific_command(latex_translated, self.translate_paragraph_latex, 'title')
 
         latex_translated = process_latex.recover_special(latex_translated)
         latex_translated = process_latex.recover_accent(latex_translated)
 
         self.close()
+
+        print(self.ntotal - self.nbad, '/',  self.ntotal, 'latex object are correctly translated')
+
         return latex_translated
